@@ -8,18 +8,17 @@ use church::pairs::{pair, fst, snd};
 use church::numerals::zero;
 use combinators::z;
 use std::ops::Index;
+use std::mem;
 
-/// Equivalent to `booleans::fls()`; produces a Church-encoded `nil`, the last link of a
-/// Church list.
+/// Equivalent to `booleans::fls()`; produces a Church-encoded `nil`, the last
+/// link of a Church list.
 ///
 /// NIL := FALSE
 ///
 /// # Example
 /// ```
 /// use lambda_calculus::church::lists::nil;
-/// use lambda_calculus::church::booleans::fls;
 ///
-/// assert_eq!(nil(), fls());
 /// assert!(!nil().is_list());
 /// assert!(nil().is_empty());
 /// ```
@@ -31,15 +30,10 @@ pub fn nil() -> Term { fls() }
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
 /// use lambda_calculus::church::lists::{nil, null};
-/// use lambda_calculus::church::booleans::tru;
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
-/// assert_eq!(beta(app!(null(), nil()), NOR, 0, false), tru());
-/// # }
+/// assert_eq!(beta(app(null(), nil()), NOR, 0, false), true.into());
 /// ```
 pub fn null() -> Term {
     abs(app!(Var(1), abs!(3, fls()), tru()))
@@ -51,33 +45,28 @@ pub fn null() -> Term {
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
-/// use lambda_calculus::church::numerals::{zero, one};
 /// use lambda_calculus::church::lists::{nil, cons};
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
-/// let list_110_consed = beta(
+/// let list_consed = beta(
 ///     app!(
 ///         cons(),
-///         one(),
+///         1.into(),
 ///         app!(
 ///             cons(),
-///             one(),
+///             2.into(),
 ///             app!(
 ///                 cons(),
-///                 zero(),
+///                 3.into(),
 ///                 nil()
 ///             )
 ///         )
 ///     ), NOR, 0, false
 /// );
-/// let list_110_from_vec = Term::from(vec![one(), one(), zero()]);
 ///
-/// assert_eq!(list_110_consed, list_110_from_vec);
-/// # }
+/// let list_from_vec = Term::from(vec![1.into(), 2.into(), 3.into()]);
+///
+/// assert_eq!(list_consed, list_from_vec);
 /// ```
 pub fn cons() -> Term { pair() }
 
@@ -87,18 +76,12 @@ pub fn cons() -> Term { pair() }
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::head;
-/// use lambda_calculus::church::numerals::{zero, one};
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
-/// let list_110 = Term::from(vec![one(), one(), zero()]);
+/// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
-/// assert_eq!(beta(app!(head(), list_110), NOR, 0, false), one());
-/// # }
+/// assert_eq!(beta(app(head(), list), NOR, 0, false), 1.into());
 /// ```
 pub fn head() -> Term { fst() }
 
@@ -109,18 +92,15 @@ pub fn head() -> Term { fst() }
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::tail;
-/// use lambda_calculus::church::numerals::{zero, one};
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
-/// let list_110 = Term::from(vec![one(), one(), zero()]);
+/// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
-/// assert_eq!(beta(app!(tail(), list_110), NOR, 0, false), Term::from(vec![one(), zero()]));
-/// # }
+/// assert_eq!(
+///     beta(app(tail(), list), NOR, 0, false),
+///     vec![2.into(), 3.into()].into()
+/// );
 /// ```
 pub fn tail() -> Term { snd() }
 
@@ -131,18 +111,13 @@ pub fn tail() -> Term { snd() }
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::{length, nil};
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
-/// let list_4 = Term::from(vec![1.into(), 1.into(), 0.into(), 1.into()]);
+/// let list = Term::from(vec![1.into(), 2.into(), 3.into(), 4.into()]);
 ///
-/// assert_eq!(beta(app!(length(), nil() ), NOR, 0, false), 0.into());
-/// assert_eq!(beta(app!(length(), list_4), NOR, 0, false), 4.into());
-/// # }
+/// assert_eq!(beta(app(length(), nil()), NOR, 0, false), 0.into());
+/// assert_eq!(beta(app(length(), list ), NOR, 0, false), 4.into());
 /// ```
 pub fn length() -> Term {
     app!(
@@ -170,20 +145,15 @@ pub fn length() -> Term {
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::reverse;
-/// use lambda_calculus::church::numerals::{zero, one};
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
-/// let list = Term::from(vec![one(), one(), zero()]);
+/// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
-/// assert_eq!(beta(app!(reverse(), list), NOR, 0, false),
-///            Term::from(vec![zero(), one(), one()])
+/// assert_eq!(
+///     beta(app(reverse(), list), NOR, 0, false),
+///     vec![3.into(), 2.into(), 1.into()].into()
 /// );
-/// # }
 /// ```
 pub fn reverse() -> Term {
     app!(
@@ -215,16 +185,13 @@ pub fn reverse() -> Term {
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::list;
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
-/// assert_eq!(beta(app!(list(), 3.into(), 0.into(), 1.into(), 1.into()), NOR, 0, false),
-///            Term::from(vec![0.into(), 1.into(), 1.into()]));
-/// # }
+/// assert_eq!(
+///     beta(app!(list(), 3.into(), 1.into(), 2.into(), 3.into()), NOR, 0, false),
+///     vec![1.into(), 2.into(), 3.into()].into()
+/// );
 /// ```
 pub fn list() -> Term {
     abs(app!(
@@ -242,19 +209,16 @@ pub fn list() -> Term {
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::append;
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
-/// let list1 = Term::from(vec![0.into(), 1.into()]);
-/// let list2 = Term::from(vec![2.into(), 3.into()]);
+/// let list1 = Term::from(vec![1.into(), 2.into()]);
+/// let list2 = Term::from(vec![3.into(), 4.into()]);
 ///
-/// assert_eq!(beta(app!(append(), list1, list2), NOR, 0, false),
-///            Term::from(vec![0.into(), 1.into(), 2.into(), 3.into()]));
-/// # }
+/// assert_eq!(
+///     beta(app!(append(), list1, list2), NOR, 0, false),
+///     vec![1.into(), 2.into(), 3.into(), 4.into()].into()
+/// );
 /// ```
 pub fn append() -> Term {
     z().app(
@@ -284,18 +248,13 @@ pub fn append() -> Term {
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::index;
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
-/// let list = Term::from(vec![3.into(), 4.into(), 5.into()]);
+/// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
-/// assert_eq!(beta(app!(index(), 0.into(), list.clone()), NOR, 0, false), 3.into());
-/// assert_eq!(beta(app!(index(), 2.into(), list        ), NOR, 0, false), 5.into());
-/// # }
+/// assert_eq!(beta(app!(index(), 0.into(), list.clone()), NOR, 0, false), 1.into());
+/// assert_eq!(beta(app!(index(), 2.into(), list        ), NOR, 0, false), 3.into());
 /// ```
 pub fn index() -> Term {
     abs!(2, app!(
@@ -313,19 +272,16 @@ pub fn index() -> Term {
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::map;
 /// use lambda_calculus::church::numerals::succ;
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
 /// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
-/// assert_eq!(beta(app!(map(), succ(), list), NOR, 0, false),
-///            Term::from(vec![2.into(), 3.into(), 4.into()]));
-/// # }
+/// assert_eq!(
+///     beta(app!(map(), succ(), list), NOR, 0, false),
+///     vec![2.into(), 3.into(), 4.into()].into()
+/// );
 /// ```
 pub fn map() -> Term {
     z().app(
@@ -352,27 +308,22 @@ pub fn map() -> Term {
 }
 
 /// Applied to a function, a starting value and a Church list it performs a
-/// [left fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists) on the
-/// list.
+/// [left fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists)
+/// on the list.
 ///
 /// FOLDL := Z (λzfsl. NULL l (λx.s) (λx.z f (f s (FST l)) (SND l)) I) =
 /// Z (λ λ λ λ NULL 1 (λ 3) (λ 5 4 (4 3 (FST 2)) (SND 2)) I)
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::{foldl, nil};
 /// use lambda_calculus::church::numerals::plus;
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
 /// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
 /// assert_eq!(beta(app!(foldl(), plus(), 0.into(), list ), NOR, 0, false), 6.into());
 /// assert_eq!(beta(app!(foldl(), plus(), 0.into(), nil()), NOR, 0, false), 0.into());
-/// # }
 /// ```
 pub fn foldl() -> Term {
     z().app(
@@ -397,27 +348,22 @@ pub fn foldl() -> Term {
 }
 
 /// Applied to a function, a starting value and a Church list it performs a
-/// [right fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists) on the
-/// list.
+/// [right fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists)
+/// on the list.
 ///
 /// FOLDR := λfsl. Z (λzt. NULL t (λx.s) (λx.f (FST t) (z (SND t))) I) l =
 /// λ λ λ Z (λ λ NULL 1 (λ 5) (λ 6 (FST 2) (3 (SND 2))) I) 1
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::{foldr, nil};
 /// use lambda_calculus::church::numerals::plus;
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
 /// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
 /// assert_eq!(beta(app!(foldr(), plus(), 0.into(), list ), NOR, 0, false), 6.into());
 /// assert_eq!(beta(app!(foldr(), plus(), 0.into(), nil()), NOR, 0, false), 0.into());
-/// # }
 /// ```
 pub fn foldr() -> Term {
     abs!(3, app!(
@@ -445,23 +391,22 @@ pub fn foldr() -> Term {
 ///
 /// # Example
 /// ```
-/// # #[macro_use] extern crate lambda_calculus;
-/// # fn main() {
-/// use lambda_calculus::term::Term;
 /// use lambda_calculus::church::lists::{filter, nil};
 /// use lambda_calculus::church::numerals::{is_zero, gt};
 /// use lambda_calculus::combinators::c;
-/// use lambda_calculus::reduction::beta;
-/// use lambda_calculus::reduction::Order::*;
+/// use lambda_calculus::*;
 ///
 /// let list = Term::from(vec![0.into(), 1.into(), 2.into(), 3.into()]);
-/// let gt1  = app!(c(), gt(), 1.into());
+/// let gt_1 = app!(c(), gt(), 1.into()); // greater than 1
 ///
-/// assert_eq!(beta(app!(filter(), is_zero(), list.clone()), NOR, 0, false),
-///            Term::from(vec![0.into()]));
-/// assert_eq!(beta(app!(filter(), gt1, list), NOR, 0, false),
-///            Term::from(vec![2.into(), 3.into()]));
-/// # }
+/// assert_eq!(
+///     beta(app!(filter(), is_zero(), list.clone()), NOR, 0, false),
+///     vec![0.into()].into()
+/// );
+/// assert_eq!(
+///     beta(app!(filter(), gt_1, list), NOR, 0, false),
+///     vec![2.into(), 3.into()].into()
+/// );
 /// ```
 pub fn filter() -> Term {
     z().app(
@@ -490,6 +435,40 @@ pub fn filter() -> Term {
     )
 }
 
+/// Applied to a Church-encoded list it returns the last element.
+///
+/// LAST := Z (λzl.NULL l (λx.NIL) (λx.NULL (TAIL l) (HEAD l) (z (TAIL l))) I) =
+/// Z (λ 2 1. NULL 1 (λ NIL) (λ NULL (TAIL 2) (HEAD 2) (3 (TAIL 2))) I)
+///
+/// # Example
+/// ```
+/// use lambda_calculus::church::lists::{last};
+/// use lambda_calculus::*;
+///
+/// let list = Term::from(vec![0.into(), 1.into(), 2.into(), 3.into()]);
+///
+/// assert_eq!(beta(app(last(), list), NOR, 0, false), 3.into());
+/// ```
+pub fn last() -> Term {
+    app(
+        z(),
+        abs!(2, app!(
+            null(),
+            Var(1),
+            abs(fls()),
+            abs(app!(
+                Var(2),
+                abs!(2, Var(1)),
+                abs!(5, Var(1)),
+                abs!(2, Var(2)),
+                app(Var(2), abs!(2, Var(2))),
+                app(Var(3), app(Var(2), abs!(2, Var(1))))
+            )),
+            abs(Var(1))
+        ))
+    )
+}
+
 impl Term {
     /// Checks whether self is a empty Church list, i.e. `nil()`.
     ///
@@ -507,7 +486,7 @@ impl Term {
     fn last_ref(&self) -> Result<&Term, Error> {
         if !self.is_pair() { return Err(NotAList) }
 
-        let mut last_candidate = try!(self.snd_ref());
+        let mut last_candidate = self.snd_ref()?;
 
         while let Ok(second) = last_candidate.snd_ref() {
             last_candidate = second;
@@ -520,12 +499,9 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert!(list_110.is_list());
+    /// assert!(Term::from(vec![1.into(), 2.into(), 3.into()]).is_list());
     /// ```
     pub fn is_list(&self) -> bool {
         self.last_ref() == Ok(&nil())
@@ -536,12 +512,12 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.uncons(), Ok((one(), Term::from(vec![one(), zero()]))));
+    /// assert_eq!(
+    ///     Term::from(vec![1.into(), 2.into(), 3.into()]).uncons(),
+    ///     Ok((1.into(), vec![2.into(), 3.into()].into()))
+    /// );
     /// ```
     /// # Errors
     ///
@@ -559,12 +535,12 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.uncons_ref(), Ok((&one(), &Term::from(vec![one(), zero()]))));
+    /// assert_eq!(
+    ///     Term::from(vec![1.into(), 2.into(), 3.into()]).uncons_ref(),
+    ///     Ok((&1.into(), &vec![2.into(), 3.into()].into()))
+    /// );
     /// ```
     /// # Errors
     ///
@@ -582,13 +558,12 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let mut list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.uncons_mut(),
-    ///            Ok((&mut one(), &mut Term::from(vec![one(), zero()]))));
+    /// assert_eq!(
+    ///     Term::from(vec![1.into(), 2.into(), 3.into()]).uncons_mut(),
+    ///     Ok((&mut 1.into(), &mut vec![2.into(), 3.into()].into()))
+    /// );
     /// ```
     /// # Errors
     ///
@@ -605,90 +580,90 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.head(), Ok(one()));
+    /// assert_eq!(
+    ///     Term::from(vec![1.into(), 2.into(), 3.into()]).head(),
+    ///     Ok(1.into())
+    /// );
     /// ```
     /// # Errors
     ///
     /// The function will return an error if `self` is not a Church list.
     pub fn head(self) -> Result<Term, Error> {
-        Ok(try!(self.uncons()).0)
+        Ok(self.uncons()?.0)
     }
 
     /// Returns a reference to the first term of a Church list.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.head_ref(), Ok(&one()));
+    /// assert_eq!(
+    ///     Term::from(vec![1.into(), 2.into(), 3.into()]).head_ref(),
+    ///     Ok(&1.into())
+    /// );
     /// ```
     /// # Errors
     ///
     /// The function will return an error if `self` is not a Church list.
     pub fn head_ref(&self) -> Result<&Term, Error> {
-        Ok(try!(self.uncons_ref()).0)
+        Ok(self.uncons_ref()?.0)
     }
 
     /// Returns a mutable reference to the first term of a Church list.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let mut list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.head_mut(), Ok(&mut one()));
+    /// assert_eq!(
+    ///     Term::from(vec![1.into(), 2.into(), 3.into()]).head_mut(),
+    ///     Ok(&mut 1.into())
+    /// );
     /// ```
     /// # Errors
     ///
     /// The function will return an error if `self` is not a Church list.
     pub fn head_mut(&mut self) -> Result<&mut Term, Error> {
-        Ok(try!(self.uncons_mut()).0)
+        Ok(self.uncons_mut()?.0)
     }
 
     /// Returns a list of all the terms of a Church list but the first one, consuming `self`.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.tail(), Ok(Term::from(vec![one(), zero()])));
+    /// assert_eq!(
+    ///     Term::from(vec![1.into(), 2.into(), 3.into()]).tail(),
+    ///     Ok(vec![2.into(), 3.into()].into())
+    /// );
     /// ```
     /// # Errors
     ///
     /// The function will return an error if `self` is not a Church list.
     pub fn tail(self) -> Result<Term, Error> {
-        Ok(try!(self.uncons()).1)
+        Ok(self.uncons()?.1)
     }
 
     /// Returns a reference to a list of all the terms of a Church list but the first one.
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.tail_ref(), Ok(&Term::from(vec![one(), zero()])));
+    /// assert_eq!(
+    ///     Term::from(vec![1.into(), 2.into(), 3.into()]).tail_ref(),
+    ///     Ok(&vec![2.into(), 3.into()].into())
+    /// );
     /// ```
     /// # Errors
     ///
     /// The function will return an error if `self` is not a Church list.
     pub fn tail_ref(&self) -> Result<&Term, Error> {
-        Ok(try!(self.uncons_ref()).1)
+        Ok(self.uncons_ref()?.1)
     }
 
     /// Returns a mutable reference to a list of all the terms of a Church list but the
@@ -696,30 +671,27 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let mut list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.tail_mut(), Ok(&mut Term::from(vec![one(), zero()])));
+    /// assert_eq!(
+    ///     Term::from(vec![1.into(), 2.into(), 3.into()]).tail_mut(),
+    ///     Ok(&mut vec![2.into(), 3.into()].into())
+    /// );
     /// ```
     /// # Errors
     ///
     /// The function will return an error if `self` is not a Church list.
     pub fn tail_mut(&mut self) -> Result<&mut Term, Error> {
-        Ok(try!(self.uncons_mut()).1)
+        Ok(self.uncons_mut()?.1)
     }
 
     /// Returns the length of a Church list
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let list_110 = Term::from(vec![one(), one(), zero()]);
-    ///
-    /// assert_eq!(list_110.len(), Ok(3));
+    /// assert_eq!(Term::from(vec![1.into(), 2.into(), 3.into()]).len(), Ok(3));
     /// ```
     /// # Errors
     ///
@@ -730,7 +702,7 @@ impl Term {
 
         while *inner != nil() {
             n += 1;
-            inner = try!(inner.tail_ref());
+            inner = inner.tail_ref()?;
         }
 
         Ok(n)
@@ -741,12 +713,16 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let list_110 = Term::from(vec![one(), one(), zero()]);
+    /// let list_from_vec = Term::from(vec![1.into(), 2.into(), 3.into()]);
+    /// let list_pushed = Term::from(vec![])
+    ///                   .push(3.into())
+    ///                   .and_then(|t| t.push(2.into()))
+    ///                   .and_then(|t| t.push(1.into()))
+    ///                   .unwrap();
     ///
-    /// assert_eq!(list_110.push(zero()), Ok(Term::from(vec![zero(), one(), one(), zero()])));
+    /// assert_eq!(list_from_vec, list_pushed);
     /// ```
     /// # Errors
     ///
@@ -761,26 +737,26 @@ impl Term {
     ///
     /// # Example
     /// ```
-    /// use lambda_calculus::term::Term;
-    /// use lambda_calculus::church::numerals::{zero, one};
+    /// use lambda_calculus::*;
     ///
-    /// let mut list_110 = Term::from(vec![one(), one(), zero()]);
+    /// let mut list = Term::from(vec![1.into(), 2.into(), 3.into()]);
     ///
-    /// assert_eq!(list_110.pop(), Ok(one()));
-    /// assert_eq!(list_110, Term::from(vec![one(), zero()]));
-    /// assert_eq!(list_110.pop(), Ok(one()));
-    /// assert_eq!(list_110, Term::from(vec![zero()]));
-    /// assert_eq!(list_110.pop(), Ok(zero()));
-    /// assert_eq!(list_110, Term::from(vec![]));
+    /// assert_eq!(list.pop(), Ok(1.into()));
+    /// assert_eq!(list, vec![2.into(), 3.into()].into());
+    /// assert_eq!(list.pop(), Ok(2.into()));
+    /// assert_eq!(list, vec![3.into()].into());
+    /// assert_eq!(list.pop(), Ok(3.into()));
+    /// assert_eq!(list, vec![].into());
     ///
-    /// assert!(list_110.is_empty())
+    /// assert!(list.is_empty())
     /// ```
     /// # Errors
     ///
     /// The function will return an error if `self` is not a Church list.
     pub fn pop(&mut self) -> Result<Term, Error> {
-        let (head, tail) = try!(self.clone().uncons()); // TODO: drop clone()?
-        *self = tail;
+        let to_uncons = mem::replace(self, Var(0)); // replace self with a dummy
+        let (head, tail) = to_uncons.uncons()?;
+        mem::replace(self, tail); // replace self with tail
 
         Ok(head)
     }
@@ -827,3 +803,57 @@ impl Index<usize> for Term {
         candidate.head_ref().unwrap() // safe - verified as valid by is_list()
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn list_length() {
+        assert_eq!(nil().len(), Ok(0));
+        assert_eq!(Term::from(vec![]).len(), Ok(0));
+        assert_eq!(Term::from(vec![1.into()]).len(), Ok(1));
+        assert_eq!(Term::from(vec![1.into(), 2.into()]).len(), Ok(2));
+        assert_eq!(Term::from(vec![1.into(), 2.into(), 3.into()]).len(), Ok(3));
+    }
+
+    #[test]
+    fn list_head_tail() {
+        let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
+
+        assert_eq!(list.head_ref(), Ok(&1.into()));
+        assert_eq!(list.tail_ref(), Ok(&vec![2.into(), 3.into()].into()));
+        assert_eq!(
+            list.tail_ref().and_then(|t| t.head_ref()),
+            Ok(&2.into())
+        );
+        assert_eq!(
+            list.tail_ref().and_then(|t| t.tail_ref()).and_then(|t| t.head_ref()),
+            Ok(&3.into())
+        );
+
+        assert_eq!(list.uncons(), Ok((1.into(), vec![2.into(), 3.into()].into())));
+    }
+
+    #[test]
+    fn iterating_list() {
+        let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
+        let mut iter = list.into_iter();
+
+        assert_eq!(iter.next(), Some(1.into()));
+        assert_eq!(iter.next(), Some(2.into()));
+        assert_eq!(iter.next(), Some(3.into()));
+        assert_eq!(iter.next(), None);
+    }
+
+    #[test]
+    fn indexing_list() {
+        let list = Term::from(vec![0.into(), 1.into(), 2.into(), 3.into(), 4.into()]);
+
+        assert_eq!(list[0], 0.into());
+        assert_eq!(list[1], 1.into());
+        assert_eq!(list[2], 2.into());
+        assert_eq!(list[3], 3.into());
+        assert_eq!(list[4], 4.into());
+    }
+}
